@@ -385,7 +385,7 @@ void randomizeDirEntryList(struct sDirEntryList *list) {
 
 	tmp=randlist;
 	last=skip;
-	while(tmp->next && ((uint8_t) tmp->next->sname[0] == DE_FREE)) {
+	while(tmp->next && ((uint8_t) tmp->next->sname[0] != DE_FREE)) {
 		last++;
 		tmp=tmp->next;
 	}
