@@ -29,6 +29,7 @@
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <locale.h>
 #include <sys/param.h>
 #include "entrylist.h"
 #include "errors.h"
@@ -41,6 +42,24 @@
 #include "stringlist.h"
 #include "mallocv.h"
 
+char *getCharSet(void) {
+/*
+	find out character set
+*/
+	char *locale,*charset;
+	
+	locale=setlocale(LC_CTYPE, NULL);
+
+	if (locale != NULL) {
+		charset = strchr(locale, '.');
+		if (charset != NULL) { 
+			return charset+1;
+		}
+	}
+	
+	return NULL;
+}
+
 int32_t parseLongFilenamePart(struct sLongDirEntry *lde, char *str, iconv_t cd) {
 /*
 	retrieves a part of a long filename from a
@@ -88,15 +107,23 @@ int32_t parseLongFilenamePart(struct sLongDirEntry *lde, char *str, iconv_t cd)
 	char *outptr = &(str[0]);
 	char *inptr = &(utf16str[0]);
 	size_t ret;
-
+	
 	while (incount != 0) {
-    if ((ret=iconv(cd, &inptr, &incount, &outptr, &outcount)) == (size_t)-1) {
-			stderror();
-			myerror("WARNING: iconv failed!");
-			break;
-    }
-  }
-	outptr[0]='\0';
+		if ((ret=iconv(cd, &inptr, &incount, &outptr, &outcount)) == (size_t)-1) {
+			if (errno == EILSEQ) {
+				outptr[0]='?';
+				outptr++;
+				incount+=2;
+				outcount++;
+				inptr+=2;
+			} else {
+				stderror();
+				myerror("WARNING: iconv failed!");
+				break;
+			}
+		}
+	}
+  	outptr[0]='\0';
 
 #endif
 
@@ -261,14 +288,15 @@ int32_t parseExFATClusterChain(struct sFileSystem *fs, struct sClusterChain *cha
 
 	struct sExFATDirEntry de;
 	struct sExFATDirEntrySet *des;
-	struct sExFATDirEntryList *del;
+	struct sExFATDirEntryList *del=NULL;
 
 	char name[MAX_PATH_LEN+1];
-	char str[32];
+	char str[31];
 	char *outptr, *inptr;
 	uint8_t nameLength=0;
 
-	size_t outcount, incount, iret;
+	size_t outcount=30;
+	size_t incount, iret;
 
 	*direntrysets=0;
 
@@ -376,7 +404,7 @@ int32_t parseExFATClusterChain(struct sFileSystem *fs, struct sClusterChain *cha
 				        }
 					outptr[0]='\0';
 
-					strncat(name, str, 30);
+					strncat(name, str, 31);
 
 					// we are done here
 					if (entries == expected_entries) {
@@ -512,7 +540,7 @@ int32_t parseClusterChain(struct sFileSystem *fs, struct sClusterChain *chain, s
 	union sDirEntry de;
 	struct sDirEntryList *lnde;
 	struct sLongDirEntryList *llist;
-	char tmp[MAX_PATH_LEN+1], dummy[MAX_PATH_LEN*2+1], sname[MAX_PATH_LEN+1], lname[MAX_PATH_LEN+1];
+	char tmp[MAX_PATH_LEN+1], dummy[MAX_PATH_LEN+1], sname[MAX_PATH_LEN+1], lname[MAX_PATH_LEN+1];
 
 	*direntries=0;
 
@@ -589,8 +617,11 @@ int32_t parseClusterChain(struct sFileSystem *fs, struct sClusterChain *chain, s
 					return -1;
 				}
 
-				snprintf(dummy, MAX_PATH_LEN*2+1, "%s%s", tmp, lname);
-				strncpy(lname, dummy, MAX_PATH_LEN);
+				strncpy(dummy, tmp, MAX_PATH_LEN);
+				dummy[MAX_PATH_LEN]='\0';
+				strncat(dummy, lname, MAX_PATH_LEN - strlen(dummy));
+				dummy[MAX_PATH_LEN]='\0';
+				strncpy(lname, dummy, MAX_PATH_LEN+1);
 
 				break;
 			default:
@@ -708,8 +739,8 @@ int32_t parseFat1xRootDirEntries(struct sFileSystem *fs, struct sDirEntryList *l
 			dummy[MAX_PATH_LEN]='\0';
 			strncat(dummy, lname, MAX_PATH_LEN - strlen(dummy));
 			dummy[MAX_PATH_LEN]='\0';
-			strncpy(lname, dummy, MAX_PATH_LEN);
-			dummy[MAX_PATH_LEN]='\0';
+			strncpy(lname, dummy, MAX_PATH_LEN+1);
+			//dummy[MAX_PATH_LEN]='\0';
 			break;
 		default:
 			myerror("Unhandled return code!");
@@ -1398,7 +1429,7 @@ int32_t sortFileSystem(char *filename) {
 
 	uint32_t mode = FS_MODE_RW;
 
-	struct sFileSystem fs = {0};
+	struct sFileSystem fs;
 
 	const char rootDir[2] = {DIRECTORY_SEPARATOR, '\0'};
 
